// Copyright (C) 2006 Alexei Miasnikov
//
// Contents: Definition of an Image class
//
// Principal Author: Alexei Miasnikov
//
// Status: in progress
//
// Revision History:
//


#ifndef _WORD_DRAW_H_
#define _WORD_DRAW_H_

#include "AImage.h"
#include "Word.h"

class WordDraw
{
public:
  WordDraw( int n, const Word& w, bool draw_grid = true ): 
    ss( 6 ),
    //    theWord(w),
    N(n),
    betBraids(0),
    theLength( 0 )
  {
    theImage = new CImage( w.length()*(6+1) + 1 , n*(6+1) + 1 );
    
    for (int i=0;i<theImage->getWidth();i++)
      for (int j=0;j<theImage->getHeight();j++){
	theImage->setRedPixel(i,j,255);
	theImage->setBluePixel(i,j,255);
	theImage->setGreenPixel(i,j,255);
      }
    
    drawCompressedBraid( w );

    if (draw_grid){
      for (int i=0;i<=(theLength+1)*(ss+1);i+=(ss+1))
	drawVerticalGrid( i, (i/(ss+1) % 10 ? 220 :0) );
      for (int i=0;i<theImage->getHeight();i+=(ss+1))
	drawHorizontalGrid( i );
    }
  
  }

  WordDraw( int n, const list<Word>& w, bool draw_grid = true ): 
    ss( 6 ),
    //    theWords(w),
    N(n),
    betBraids(20),
    theLength( 0 )
  {
    int max_length = 0;
    for (list<Word>::const_iterator I=w.begin();I!=w.end();I++)
      if (max_length  < I->length())
	max_length = I->length();
    
    int one_braid_width = n*(6+1)+1;
    theImage = new CImage( max_length*(6+1) + 1 , w.size()*one_braid_width + 1 + (w.size()-1)*betBraids );
      
    for (int i=0;i<theImage->getWidth();i++)
      for (int j=0;j<theImage->getHeight();j++){
	theImage->setRedPixel(i,j,255);
	theImage->setBluePixel(i,j,255);
	theImage->setGreenPixel(i,j,255);
      }
    
    int i_offset=0;
    for (list<Word>::const_iterator I=w.begin();I!=w.end();I++,i_offset+=one_braid_width+betBraids)
      drawCompressedBraid( *I, i_offset );

    if (draw_grid){
      for (int i=0;i<=(theLength+1)*(ss+1);i+=(ss+1))
	drawVerticalGrid( i , ( i/(ss+1) % 10 ? 220 : 0 ));
      
      i_offset=one_braid_width;
      for (int i=0;i<theImage->getHeight();i+=(ss+1)){
	if ( i >= i_offset){
	  i+=betBraids-ss;
	  i_offset+=one_braid_width+betBraids;
	}
	drawHorizontalGrid( i );
      }
    }
  
  }
  ~WordDraw() { delete theImage; }
  void saveTo( const string& f_name ) {theImage->saveTo(f_name);}
private:
  //  void drawBraid(){
    
  //    int i=0;
  //    for (ConstWordIterator wI=theWord.begin();wI!=theWord.end();wI++,i++)
  //      drawGenerator( *wI,i );    
  //  }
  
  
  void drawCompressedBraid( const Word& theWord, int vert_offset = 0){
    vector<int> positions(N+2,-1); // +2 for the left and right padding
    
    int i=0;
    for (ConstWordIterator wI=theWord.begin();wI!=theWord.end();wI++,i++){
      Generator g = *wI;
      int absg = abs(g);
      int pos = max(positions[absg],max(positions[absg-1],positions[absg+1])) + 1;
      drawGenerator( g,pos,vert_offset );
      positions[absg] = pos;
      if (theLength < pos) theLength = pos;
    }
    
  }
  
  void drawGenerator( Generator g, int pos, int vert_offset ){
    for (int iss=0;iss<ss;iss++)
      for (int jss=0;jss<ss;jss++)
	if ( g > 0 ){
	  theImage->setRedPixel( pos*ss + iss + pos+1,  vert_offset+ g + (g - 1)*ss  + jss,168);
	  theImage->setBluePixel( pos*ss + iss + pos+1, vert_offset+ g + (g - 1)*ss  + jss,168);
	  theImage->setGreenPixel( pos*ss + iss + pos+1, vert_offset+g + (g - 1)*ss  + jss,168);
	} else {
	  int ag = abs(g);
	  theImage->setRedPixel( pos*ss +iss + pos+1,  vert_offset+ ag + (ag-1)*ss + jss,0); 
	  theImage->setBluePixel( pos*ss +iss + pos+1, vert_offset+ ag + (ag-1)*ss + jss,0); 
	  theImage->setGreenPixel( pos*ss +iss + pos+1,vert_offset+ ag + (ag-1)*ss + jss,0); 
	}
  }
  
  void drawVerticalGrid( int vpos, int color){
    
    for (int i=0;i<theImage->getHeight();i+=2){
   	  theImage->setRedPixel( vpos, i,color); 
	  theImage->setBluePixel( vpos, i, color );
	  theImage->setGreenPixel( vpos, i, color );
    }
  }
  
  void drawHorizontalGrid( int hpos){
    for (int i=0;i<=(theLength+1)*(ss+1);i+=2){
   	  theImage->setRedPixel( i, hpos,220); 
	  theImage->setBluePixel( i, hpos, 220 );
	  theImage->setGreenPixel( i, hpos, 220 );
    }
  }
  
  //  const Word& theWord;
  int N;
  CImage* theImage;
  int ss;
  int theLength;
  int betBraids;
};

#endif
